[manpage_begin central n 1.0b1]
[require   Tcl 8.6]
[require   central [opt 1.0b1]]
[moddesc   {Central}]
[titledesc {Central}]
[category  {Programming tools}]
[copyright {2020 Stuart Cassoff <stwo@users.sourceforge.net>}]
[keywords  central pkgconfig]

[description]

	Central is a passive registration system for Tcl extensions and programs,
	based on the idea and workings of the Tcl [cmd {pkgconfig}] command.
	[para]
	To participate, a [cmd {pkgconfig}] command should exist in the namespace
	of an extension or program ([term {"package"}]).
	Central itself is not needed by a providing package, only by a querying program.
	[para]
	Central can be used by a program to scan the namespace hierarchy ([cmd {discover}]),
	and query any package information found.
	[para]
	The [cmd {pkgconfig}] command provided by a package should
	behave like the Tcl [cmd {pkgconfig}] command.
	[para]
	Central makes no effort to interpret the information returned by a
	[cmd {pkgconfig}] command; this information is entirely up to the package.
	[para]
	Central is all about "about": allowing a program to easily
	provide information about itself to the outside world.
	[para]
	The usage example files, the program centraltool.tcl, and
	the Central extension itself contain [cmd {pkgconfig}] commands and other
	code bits that can be copied or serve as examples and should make it easy
	to add Central functionality to extensions or programs.
	[para]
	Central works with Jim as well as with Tcl.

[section {COMMANDS}]

[usage namespace import ::central::central]

[list_begin definitions]

	[call [cmd {central discover}] [opt [arg {exclude}]]]
		Discovers packages. Ignores packages specified in [opt [arg {exclude}]].
		Returns a list of all discovered packages.

	[call [cmd {central packages}]]
		Returns a list of previously discovered packages.

	[call [cmd {central packagekeys}] [arg {package}]]
		Returns a list of all keys for package [arg {package}].

	[call [cmd {central package}] [opt [arg {key ...}]]]
		Returns all package info or info matching [opt [arg {key}]].

[list_end]


[section EXAMPLES]

[example {
	# Discover packages and display package info.

	package require central
	namespace import ::central::central
	central discover
	foreach package [central packages] {
		foreach key [central packagekeys $package] {
			puts [string cat $key " -> " [central package $package $key]]
		}
	}
}]


[see_also central::extras Tcl_RegisterConfig(3)]


[manpage_end]
