#! /bin/sh
# \
exec tclsh${TCL} "$0" "$@"

#
# Central Example - Using Central #1
#
# Stuart Cassoff
#
# Fall 2020
# Version 1.0
#


#
# Acquire and retrieve info.
#
# Demonstrates: API, pattern matching, using the returned dict.
#


package require central

namespace eval central { namespace eval examples { namespace eval usage1 {

namespace eval pkgconfig {
	variable pkgconfig [dict create {*}{
		name	usage1
		title	Usage1
		descr	{Using Central #1}
		author	{Stuart Cassoff}
		when	{Fall 2020}
		version	1.0
		example	yes
	}]
	dict set pkgconfig license,type ISC
	dict set pkgconfig license,text [regsub -all {^\s+|\s+$|\t} {
		Copyright (c) 2020 Stuart Cassoff <stwo@users.sourceforge.net>

		Permission to use, copy, modify, and distribute this software for any
		purpose with or without fee is hereby granted, provided that the above
		copyright notice and this permission notice appear in all copies.

		THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
		WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
		MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
		ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
		WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
		ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
		OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
	} {}]
	proc list {} { variable pkgconfig; return [dict keys $pkgconfig] }
	proc get {key} {
		variable pkgconfig
		if {[dict exists $pkgconfig $key]} {
			return [dict get $pkgconfig $key]
		} else {
			return -code error "key \"$key\" not known"
		}
	}
	namespace export get list
	namespace ensemble create
}; # pkgconfig


proc go {{l ""}} {
	namespace import ::central::central

	## The default for patterns is "*".


	## Acquire package info
	#
	central discover

	# API
	#
	puts "Packages -> [central packages]"


	## Get all packages info as a dict
	#
	# Returns a dict {package {key value}}
	#
	# (same as [central package * *])
	#
	set pkgs [central package]

	# Dict
	#
	puts "Packages => [dict keys $pkgs]"

	puts ""


	## Use the ::tcl package for these examples
	#
	set pkg ::tcl
	puts "Using package $pkg"

	puts ""


	## Package, Key
	#
	# Returns a single value
	#
	set key threaded

	# API
	#
	puts "$pkg ($key) -> [central package $pkg $key]"

	# Dict
	#
	puts "$pkg ($key) => [dict get $pkgs $pkg $key]"

	puts ""


	## Package, Wildcard Key
	#
	# Returns a dict {key value}
	#
	set key scriptdir,*

	# API
	#
	puts "$pkg ($key) -> [central package $pkg $key]"

	# Dict
	#
	puts "$pkg ($key) => [dict filter [dict get $pkgs $pkg] key $key]"

	puts ""


	## Wildcard Package, (Wildcard) Key
	#
	# Returns a dict {package {key value}}
	#
	set pkg *
	set key de*
	puts "Using package $pkg"

	puts ""

	# API
	#
	puts "$pkg ($key) -> [central package $pkg $key]"

	# Dict
	#
	puts "$pkg ($key) => [dict map {k v} [dict filter $pkgs key $pkg] {
		set d [dict filter $v key $key]
		if {[dict size $d] == 0} { continue }
		set d
	}]"
	unset d



if 0 {

	# A package "name" is its namespace
	#
	foreach package $packages {

		puts "\nPackage/Namespace -> $package"

		# Package keys are determined by the package
		#
		set keys [central packagekeys $package]

		puts "Keys -> $keys"

		# Use [central package] to retrieve package info bits
		#
		foreach key $keys {
			if {$l ne "l" && [string match license,* $key]} { continue }
			puts [string cat $key " -> " [central package $package $key]]
		}
	}
}
}

}}}; # namespace central::examples::usage1


if {[info exists ::argv0] && [info script] eq $::argv0} {
	central::examples::usage1::go {*}$::argv
}


# EOF
