#! /bin/sh
# \
exec jimsh "$0" "$@"

#
# Central Example - Using Central with Jim
#
# Same as usage2.tcl but with Jim
#
# Stuart Cassoff
#
# Summer 2022
# Version 1.0
#


#
# Basic usage: acquire and retrieve info.
#
# Demonstrates: API, pattern matching.
#
#
# Call with 'l' to also output license info as well as package info.
#


package require central

namespace eval central { namespace eval examples { namespace eval usagej {

proc pkgconfig {subcommand args} {
	if {$subcommand ni [list get list]} {
		return -code error "unknown subcommand \"$subcommand\": must be get, or list"
	}
	set pkgconfig [dict create {*}{
		name	usagej
		title	Usagej
		descr	{Using Central with Jim}
		author	{Stuart Cassoff}
		when	{Summer 2022}
		version	1.0
		example	yes
	}]
	dict set pkgconfig license,type ISC
	dict set pkgconfig license,text [regsub -all {^\s+|\s+$|\t} {
		Copyright (c) 2018,2022 Stuart Cassoff <stwo@users.sourceforge.net>

		Permission to use, copy, modify, and distribute this software for any
		purpose with or without fee is hereby granted, provided that the above
		copyright notice and this permission notice appear in all copies.

		THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
		WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
		MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
		ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
		WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
		ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
		OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
	} {}]
	return -code {*}[expr {
	    $subcommand eq "list"
		? [llength $args] == 0
		    ? [list ok [dict keys $pkgconfig]]
		    : [list error "wrong # args: should be \"[namespace current]::pkgconfig list\""]
		: $subcommand eq "get"
		    ? [llength $args] == 1
			? [dict exists $pkgconfig [set key [lindex $args 0]]]
			    ? [list ok [dict get $pkgconfig $key]]
			    : [list error "key \"$key\" not known"]
			: [list error "wrong # args: should be \"[namespace current]::pkgconfig get key\""]
		    : [list uh oh]
	}]
}; #pkgconfig


proc go {{l ""}} {
	namespace import ::central::central

	# Acquire package info
	#
	# Either way the result is the same
	#
	if 0 {
		central discover
		set packages [central packages]
	} else {
		set packages [central discover]
	}

	puts "Packages -> $packages"

	# A package "name" is its namespace
	#
	foreach package $packages {

		puts "\nPackage/Namespace -> $package"

		# Package keys are determined by the package
		#
		set keys [central packagekeys $package]

		puts "Keys -> $keys"

		# Use [central package] to retrieve package info bits
		#
		foreach key $keys {
			if {$l ne "l" && [string match license,* $key]} { continue }
			puts [string cat $key " -> " [central package $package $key]]
		}
	}
}

}}}; # namespace central::examples::usagej


if {[info exists ::argv0] && [info script] eq $::argv0} {
	central::examples::usagej::go {*}$::argv
}


# EOF
