#
# Central - Central Coordination Bureau
#
# Stuart Cassoff
#
# Summer 2018
# Version 0.1
#
# Fall 2020
# Version 1.0b1
#


namespace eval central {

if {$::tcl_platform(engine) eq "Jim"} {

proc pkgconfig {subcommand args} {
	if {$subcommand ni [list get list]} {
		return -code error "unknown subcommand \"$subcommand\": must be get, or list"
	}
	set pkgconfig [dict create {*}{
		name	central
		title	Central
		descr	{Central Coordination Bureau}
		author	{Stuart Cassoff}
		when	{Fall 2020}
		version	1.0b1
	}]
	dict set pkgconfig license,type ISC
	dict set pkgconfig license,text [regsub -all {^\s+|\s+$|\t} {
		Copyright (c) 2018,2020 Stuart Cassoff <stwo@users.sourceforge.net>

		Permission to use, copy, modify, and distribute this software for any
		purpose with or without fee is hereby granted, provided that the above
		copyright notice and this permission notice appear in all copies.

		THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
		WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
		MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
		ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
		WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
		ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
		OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
	} {}]
	return -code {*}[expr {
	    $subcommand eq "list"
		? [llength $args] == 0
		    ? [list ok [dict keys $pkgconfig]]
		    : [list error "wrong # args: should be \"[namespace current]::pkgconfig list\""]
		: $subcommand eq "get"
		    ? [llength $args] == 1
			? [dict exists $pkgconfig [set key [lindex $args 0]]]
			    ? [list ok [dict get $pkgconfig $key]]
			    : [list error "key \"$key\" not known"]
			: [list error "wrong # args: should be \"[namespace current]::pkgconfig get key\""]
		    : [list uh oh]
	}]
}; #pkgconfig

} else { # !Jim

package require Tcl 8.6

namespace eval pkgconfig {
	variable pkgconfig [dict create {*}{
		name	central
		title	Central
		descr	{Central Coordination Bureau}
		author	{Stuart Cassoff}
		when	{Fall 2020}
		version	1.0b1
	}]
	dict set pkgconfig license,type ISC
	dict set pkgconfig license,text [regsub -all {^\s+|\s+$|\t} {
		Copyright (c) 2018,2020 Stuart Cassoff <stwo@users.sourceforge.net>

		Permission to use, copy, modify, and distribute this software for any
		purpose with or without fee is hereby granted, provided that the above
		copyright notice and this permission notice appear in all copies.

		THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
		WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
		MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
		ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
		WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
		ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
		OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
	} {}]
	proc list {} { variable pkgconfig; return [dict keys $pkgconfig] }
	proc get {key} {
		variable pkgconfig
		if {[dict exists $pkgconfig $key]} {
			return [dict get $pkgconfig $key]
		} else {
			return -code error "key \"$key\" not known"
		}
	}
	namespace export get list
	namespace ensemble create
}; # pkgconfig

}; # Jim/Tcl


## Central

variable packages [list]

proc packages {} {
	variable packages
	return $packages
}

proc packagekeys {package} {
	return [${package}::pkgconfig list]
}

proc package {{package {*}} {key {*}}} {
	variable packages
	set globbies {[*?[\]\\]}
	set pkgexact [expr {![regexp $globbies $package]}]
	set keyexact [expr {![regexp $globbies $key]}]
	set pkgs [lsearch -glob -inline -all $packages $package]
	if {$pkgexact && [llength $pkgs] == 0} {
		return -code error "package \"$package\" not known"
	}
	set res [dict create]
	foreach pkg $pkgs {
		set keys [lsearch -glob -inline -all [packagekeys $pkg] $key]
		if {[llength $keys] > 0} {
			foreach k $keys {
				dict set res $pkg $k [${pkg}::pkgconfig get $k]
			}
		} elseif {$pkgexact && $keyexact} {
			return -code error "key \"$key\" not known"
		}
	}
	if {$pkgexact && $keyexact} {
		return [dict get $res [lindex $pkgs 0] [lindex $keys 0]]
	} elseif {$pkgexact} {
		return [dict get $res [lindex $pkgs 0]]
	} else {
		return $res
	}
}

proc discover {{exclude {}}} {
	variable packages
	if {$::tcl_platform(engine) eq "Jim"} {
		set packages [list]
		foreach p [info commands ::*::pkgconfig] {
			set ns [namespace parent $p]
			if {$ns ni $exclude} {
				lappend packages $ns
			}
		}
	} else {
		set packages [list]
		set nsl [list ::]
		while {[llength $nsl] > 0} {
			set nsl [lassign $nsl[set nsl {}] ns]
			if {$ns in $exclude} continue
			if {[llength [info commands ${ns}::pkgconfig]] == 1} {
				lappend packages $ns
			}
			lappend nsl {*}[namespace children $ns]
		}
	}
	return $packages
}


## Setup
if {$::tcl_platform(engine) eq "Jim"} {
	proc central {cmd args} {
		if {$cmd in [list discover package packagekeys packages]} {
			tailcall ::central::$cmd {*}$args
		} else {
			return -code error "unknown subcommand \"$cmd\": must be discover, package, packagekeys, or packages"
		}
	}
} else {
	namespace ensemble create -prefixes no -command central -subcommands [list discover package packagekeys packages]
}

namespace export central

# Careful!
::package provide central 1.0b1


}; # namespace eval central


# EOF
